@@ -25,29 +25,11 @@
 #endif // _MSC_VER
 
 #ifndef _WIN32
-#error This is a MSVC header only.
+#error "This is a Windows header only"
 #endif
 
-// use std::thread in MSVC11 (2012) or newer
-#if _MSC_VER >= 1700
-#define USE_STD_THREAD 1
-// otherwise use boost threads
-#else
-#define USE_BOOST_THREAD 1
-#endif
-
-#ifndef TARGET_WIN_XP
-#define TARGET_WIN_XP 1
-#endif
-
-#if TARGET_WIN_XP
-#ifndef WINVER
-#define WINVER 0x0501
-#endif
-#ifndef _WIN32_WINNT
-#define _WIN32_WINNT 0x0501
-#endif
-#endif
+// Something that defines PRId64 is required to build
+#define HAVE_INTTYPES_H 1
 
 #ifndef _WIN32_WINNT
 #define _WIN32_WINNT 0x0601
@@ -58,26 +40,14 @@
 #define SIGNED_RIGHT_SHIFT_IS 1
 #endif
 
+#ifndef __MINGW32__
 #pragma warning(disable : 4996) // Deprecated posix name.
+#endif
 
-#define VERSION "0.9.3"
 #define HAVE_GETTIMEOFDAY 1
 #define HAVE_SYS_STAT_H 1
 
-#ifdef HAVE_STDINT_H
 #include <stdint.h>
-#else
-#include <boost/cstdint.hpp>
-
-typedef boost::int64_t int64_t;
-typedef boost::uint64_t uint64_t;
-typedef boost::int32_t int32_t;
-typedef boost::uint32_t uint32_t;
-typedef boost::int16_t int16_t;
-typedef boost::uint16_t uint16_t;
-typedef boost::int8_t int8_t;
-typedef boost::uint8_t uint8_t;
-#endif
 
 #include <thrift/transport/PlatformSocket.h>
 #include <thrift/windows/GetTimeOfDay.h>
@@ -89,11 +59,32 @@ typedef boost::uint8_t uint8_t;
 // windows
 #include <Winsock2.h>
 #include <ws2tcpip.h>
-#ifdef _WIN32_WCE
-#pragma comment(lib, "Ws2.lib")
+
+#ifndef __MINGW32__
+  #ifdef _WIN32_WCE
+  #pragma comment(lib, "Ws2.lib")
+  #else
+  #pragma comment(lib, "Ws2_32.lib")
+  #pragma comment(lib, "gdi32.lib") // For static OpenSSL
+  #pragma comment(lib, "crypt32.lib") // For static OpenSSL
+  #pragma comment(lib, "user32.lib") // For static OpenSSL
+  #pragma comment(lib, "advapi32.lib") // For security APIs in TPipeServer
+  #pragma comment(lib, "Shlwapi.lib")  // For StrStrIA in TPipeServer
+  #endif
+#endif // __MINGW32__
+
+// Replicate the logic of afunix.h on Windows (the header is only present on
+// newer Windows SDKs)
+#ifdef HAVE_AF_UNIX_H
+#include <afunix.h>
 #else
-#pragma comment(lib, "Ws2_32.lib")
-#pragma comment(lib, "advapi32.lib") // For security APIs in TPipeServer
-#pragma comment(lib, "Shlwapi.lib") // For StrStrIA in TPipeServer
+#ifndef UNIX_PATH_MAX
+#define UNIX_PATH_MAX 108
 #endif
+typedef struct sockaddr_un {
+  ADDRESS_FAMILY sun_family;    // AF_UNIX
+  char sun_path[UNIX_PATH_MAX]; // pathname
+} SOCKADDR_UN, *PSOCKADDR_UN;
+#endif // HAVE_AF_UNIX_H
+
 #endif // _THRIFT_WINDOWS_CONFIG_H_
